"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// @ts-ignore
const ansiStyles1 = require("ansi-styles");
const ansiStyles = ansiStyles1;
const util_1 = require("../main/util");
const util_2 = require("util");
const default_output_impl_1 = require("../main/default-output-impl");
exports.logLevels = [
    'NONE',
    'DEBUG',
    'INFO',
    'WARN',
    'ERROR'
];
const colors = {
    DEBUG: ansiStyles.color.magentaBright,
    WARN: ansiStyles.color.yellowBright,
    ERROR: [ansiStyles.color.whiteBright, ansiStyles.bgColor.bgRedBright],
};
const defaultConfig = {
    logLevels: exports.logLevels,
    colors,
    outputFormat: process.env.KARHU_JSON ? 'json' : 'text',
    formatters: {
        text: (toLog, logLevel, context, config, colorStart, colorEnd, transport) => {
            const correlationId = config.getCorrelationId && config.getCorrelationId();
            const correlationIdPart = correlationId ? ` [correlation-id=${correlationId}]` : '';
            return `[${(transport.formatNow || config.formatNow)(config)}] ${colorStart}${logLevel} ${context} ${toLog.map(val => util_1.karhuInspect(val)).join(' ')}${correlationIdPart}${colorEnd}`;
        },
        json: (toLog, logLevel, context, config, colorStart, colorEnd, transport) => {
            const correlationId = config.getCorrelationId && config.getCorrelationId();
            const output = { timestamp: (transport.formatNow || config.formatNow)(config), context, logLevel, details: toLog, correlationId };
            try {
                return JSON.stringify(output);
            }
            catch (err) {
                const fixed = Object.assign(Object.assign({}, output), { details: util_2.inspect(output.details, { depth: 5, breakLength: Infinity }) });
                return JSON.stringify(fixed);
            }
        }
    },
    contextSpecificLogLevels: new Map(),
    defaultLogLevel: 'INFO',
    envVariablePrefix: 'KARHU',
    outputMapper: value => value,
    transports: {
        console: default_output_impl_1.default
    },
    formatNow: () => new Date().toISOString()
};
exports.default = defaultConfig;
