"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
let temporarilyDisabled = false;
function toggleForceCaptureDisabled(disabled) {
    temporarilyDisabled = disabled;
}
exports.toggleForceCaptureDisabled = toggleForceCaptureDisabled;
function enableStandardOutputCapture(karhu, stdoutLogLevel, stderrLogLevel) {
    const wrappers = [
        wrap(karhu, 'stdout', stdoutLogLevel),
        wrap(karhu, 'stderr', stderrLogLevel),
    ];
    return {
        abort: () => {
            for (const wrapper of wrappers) {
                wrapper.abort();
            }
        }
    };
}
exports.enableStandardOutputCapture = enableStandardOutputCapture;
function wrap(karhu, stream, logLevel) {
    const original = process[stream].write;
    const streamAsAny = process[stream];
    streamAsAny.write = (arg1, arg2, arg3) => {
        const data = typeof arg1 === 'string' ? arg1 : arg1.toString('UTF-8');
        const callback = typeof arg2 === 'function' ? arg2 : arg3;
        if (temporarilyDisabled || data.length === 0) {
            return original.call(process[stream], arg1, arg2, arg3);
        }
        const logFunction = karhu[logLevel];
        logFunction(data.endsWith && data.endsWith('\n') ? data.substr(0, data.length - 1) : data);
        if (typeof callback === 'function') {
            callback();
        }
        return true;
    };
    return {
        abort: () => {
            process[stream].write = original;
        }
    };
}
