import { Color } from '../ansi-styles';
export declare type LogLevel = string;
export declare type KarhuTransportFn = (toLog: any, logLevel: string, context: string, config: KarhuConfig) => void;
export interface KarhuTransportMap {
    [logLevel: string]: KarhuTransportFn;
}
export interface KarhuReconfigurable {
    colors: {
        [logLevel: string]: Color | Color[];
    };
    outputFormat: string;
    formatters: {
        [outputFormat: string]: (toLog: any[], logLevel: string, context: string, config: KarhuConfig, colorStart: string, colorEnd: string, transport: KarhuTransport) => any;
    };
    defaultLogLevel: LogLevel;
    outputMapper: (value: any, logLevel: string, context: string, toLog: any[], index: number) => any;
    formatNow: (config: KarhuConfig) => string | number;
}
export interface KarhuTransport extends Partial<KarhuReconfigurable> {
    supportsColor: boolean | (() => boolean);
    outputImpl: KarhuTransportMap;
}
export interface LogLevelMap {
    [context: string]: LogLevel;
}
export interface KarhuConfig extends KarhuReconfigurable {
    logLevels: string[];
    contextSpecificLogLevels: Map<string | RegExp, LogLevel> | LogLevelMap;
    envVariablePrefix: string;
    transports: {
        [name: string]: KarhuTransport;
    };
    getCorrelationId?: () => string | undefined;
}
export declare type LogFunction = (...toLog: any[]) => void;
export interface KarhuLogger {
    error: LogFunction;
    ERROR: LogFunction;
    warn: LogFunction;
    WARN: LogFunction;
    info: LogFunction;
    INFO: LogFunction;
    debug: LogFunction;
    DEBUG: LogFunction;
}
export declare function configure(config: null | KarhuConfig): void;
export interface Karhu<LogImpl> {
    context: (context: string) => LogImpl;
    reconfigure: (newConfig: Partial<KarhuConfig>) => void;
    getConfig: () => KarhuConfig;
}
export declare const context: (activeContext: string) => KarhuLogger;
export declare function getGlobalConfig(): KarhuConfig | undefined;
export declare function usingConfig<LogImpl = KarhuLogger>(configSource: KarhuConfig | (() => KarhuConfig)): Karhu<LogImpl>;
export declare const captureStandardOutput: (logger: KarhuLogger, stdoutLogLevel?: string, stderrLogLevel?: string) => {
    abort: () => void;
};
