"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var default_1 = require("../config/default");
var util_1 = require("./util");
var capturer_1 = require("./capturer");
var noColor = {
    open: '',
    close: ''
};
var globalConfig;
if (!globalConfig)
    globalConfig = loadConfig();
function loadConfig(newConfig) {
    if (newConfig === void 0) { newConfig = null; }
    var config = newConfig || default_1.default;
    if (!config.formatters[config.outputFormat])
        throw new Error('There is no formatter for chosen output format');
    return config;
}
function configure(config) {
    globalConfig = loadConfig(config);
}
exports.configure = configure;
exports.context = function (activeContext) { return usingConfig(function () { return required(globalConfig); }).context(activeContext); };
function getGlobalConfig() {
    return globalConfig;
}
exports.getGlobalConfig = getGlobalConfig;
function usingConfig(configSource) {
    var config = typeof configSource === 'function' ? configSource() : configSource;
    return {
        context: forContext,
        reconfigure: reconfigure,
        getConfig: function () { return config; }
    };
    function forContext(activeContext) {
        var impl = {};
        var _loop_1 = function (type) {
            impl[type] = function () {
                var toLog = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    toLog[_i] = arguments[_i];
                }
                return logEvent(config, activeContext, type, toLog);
            };
            var typeLower = type.toLowerCase();
            if (!impl[typeLower]) {
                impl[typeLower] = function () {
                    var toLog = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        toLog[_i] = arguments[_i];
                    }
                    return logEvent(config, activeContext, type, toLog);
                };
            }
        };
        for (var _i = 0, _a = config.logLevels; _i < _a.length; _i++) {
            var type = _a[_i];
            _loop_1(type);
        }
        return impl;
    }
    function reconfigure(newConfig) {
        if (newConfig.logLevels && !util_1.logLevelsMatch(config.logLevels, newConfig.logLevels)) {
            throw new Error('Log levels cannot be updated with reconfigure');
        }
        Object.assign(config, newConfig);
    }
}
exports.usingConfig = usingConfig;
function logEvent(config, activeContext, logLevel, toLog) {
    var eventLogPrio = config.logLevels.indexOf(logLevel);
    var _loop_2 = function (transportName) {
        var transport = config.transports[transportName];
        var activeLogLevelPrio = config.logLevels.indexOf(getLogLevel(config, transport, activeContext));
        if (eventLogPrio < activeLogLevelPrio)
            return "continue";
        var colorEnabled = isColorEnabled(config, transport), color = colorEnabled ? getConfigColor(config, transport, logLevel) || noColor : noColor, openColor = asArray(color).map(function (c) { return c.open; }).join(''), closeColor = asArray(color).reverse().map(function (c) { return c.close; }).join(''), outputMapper = transport.outputMapper || config.outputMapper, mappedValues = toLog.map(function (value, i) { return outputMapper(value, logLevel, activeContext, toLog, i); }), outputImpl = transport.outputImpl[logLevel] || transport.outputImpl.default;
        if (!outputImpl)
            throw new Error('Transport ' + transportName + ' does not support log level ' + logLevel + ' or default');
        var outputFormat = transport.outputFormat || config.outputFormat;
        var formatter = (transport.formatters || {})[outputFormat] || config.formatters[outputFormat];
        var formatted = formatter(mappedValues, logLevel, activeContext, config, openColor, closeColor, transport);
        capturer_1.toggleForceCaptureDisabled(true);
        outputImpl(formatted, logLevel, activeContext, config);
        capturer_1.toggleForceCaptureDisabled(false);
    };
    for (var _i = 0, _a = Object.keys(config.transports); _i < _a.length; _i++) {
        var transportName = _a[_i];
        _loop_2(transportName);
    }
}
function getConfigColor(config, transport, logLevel) {
    var colors = transport.colors || config.colors;
    return colors[logLevel] || colors.default;
}
function getLogLevel(config, transport, activeContext) {
    return getOverrideLogLevel(config, activeContext) || getContextSpecificOverrideFromContext() || getOverrideLogLevel(config, null) || transport.defaultLogLevel || config.defaultLogLevel;
    function getContextSpecificOverrideFromContext() {
        if (!(config.contextSpecificLogLevels instanceof Map)) {
            return config.contextSpecificLogLevels[activeContext];
        }
        var perfectOverride = config.contextSpecificLogLevels.get(activeContext);
        if (perfectOverride)
            return perfectOverride;
        for (var _i = 0, _a = Array.from(config.contextSpecificLogLevels.keys()); _i < _a.length; _i++) {
            var key = _a[_i];
            if (key instanceof RegExp && key.test(activeContext)) {
                return config.contextSpecificLogLevels.get(key);
            }
        }
        return perfectOverride;
    }
}
function getOverrideLogLevel(config, activeContext) {
    var prefix = config.envVariablePrefix + '_LOG_LEVEL';
    return process.env[!activeContext ? prefix : prefix + '_' + toEnv(activeContext)];
}
function toEnv(activeContext) {
    return activeContext.replace(/[^a-zA-Z0-9]+/g, '_');
}
function required(val) {
    if (!val)
        throw new Error('Required value missing');
    return val;
}
function asArray(inVal) {
    if (inVal instanceof Array)
        return inVal;
    return [inVal];
}
function isColorEnabled(config, transport) {
    var override = process.env[config.envVariablePrefix + '_COLOR'];
    if (override === '0' || override === 'false')
        return false;
    if (override === '1' || override === 'true')
        return true;
    return typeof transport.supportsColor === 'function' ? transport.supportsColor() : transport.supportsColor;
}
exports.captureStandardOutput = function (logger, stdoutLogLevel, stderrLogLevel) {
    if (stdoutLogLevel === void 0) { stdoutLogLevel = 'INFO'; }
    if (stderrLogLevel === void 0) { stderrLogLevel = 'ERROR'; }
    if (!logger)
        throw new Error('Please provide a logger in the proper context to captureStandardOutput');
    return capturer_1.enableStandardOutputCapture(logger, stdoutLogLevel, stderrLogLevel);
};
