'use strict'

const severities = [
  'DEBUG',
  'INFO',
  'NOTICE',
  'WARN',
  'ERROR',
  'LOG',
  'NONE' // not used for logging events, but makes it so that setting log level to NONE outputs nothign
]


const ansi = {
  yellow: '\u001b[1m\u001b[33m',
  blue: '\u001b[1m\u001b[34m',
  magenta: '\u001b[1m\u001b[35m',
  green: '\u001b[1m\u001b[32m',
  red: '\u001b[1m\u001b[41m',
  reset: '\u001b[0m'
}

const colours = {
  DEBUG: ansi.magenta,
  WARN: ansi.yellow,
  ERROR: ansi.red,
  NOTICE: ansi.blue
}

function getLogLevel(context) {
  return process.env['NODE_LOG_LEVEL_' + context.replace(/-/g, '_')] || process.env.NODE_LOG_LEVEL || 'INFO'
}

class Logger {
  constructor(context) {
    this.context = context
  }

  log(...details) {
  logEvent('LOG', this.context, ...details)
}

info(...details) {
  logEvent('INFO', this.context, ...details)
}

notice(...details) {
  logEvent('NOTICE', this.context, ...details)
}

warn(...details) {
  logEvent('WARN', this.context, ...details)
}

warning(...details) {
  this.warn(...details)
}

error(...details) {
  logEvent('ERROR', this.context, ...details)
}

debug(...details) {
  logEvent('DEBUG', this.context, ...details)
}
}

exports.for = context => new Logger(context)


function logEvent(severity, context, ...details) {
  if (severities.indexOf(severity) < severities.indexOf(getLogLevel(context))) return
  const colour = process.stdout.isTTY && colours[severity] || '',
    reset = process.stdout.isTTY ? ansi.reset : ''
  console.log(colour + new Date().toISOString(), severity, context, ...details, reset) // eslint-disable-line no-console
}
